@@ -3,8 +3,8 @@ Miscellaneous helper functions.
 """
 
 import numpy as np
-import math
-from scipy.spatial.distance import cdist
+import math, re
+from io import BytesIO
 
 #### python 3.6 or earlier doesn't have importlib.resources, but it's backported as importlib_resources
 try:
@@ -406,7 +406,6 @@ def scale_nmr_shifts(ensemble, symmetrical_atom_numbers=None, scaling_factors="d
 #            symbol_dict = dict(zip(atomic_numbers,atomic_symbols))
             all_labels = [ f"{current_symbol}{atom_number}" for current_symbol,atom_number in zip(atomic_symbols,atom_numbers) ]
             all_labels = cctk.OneIndexedArray(all_labels)
-            label_dict = dict(zip(atom_numbers,all_labels))
 
             # check symmetrical atom numbers make sense
             n_atoms = len(atomic_numbers)
@@ -606,3 +605,104 @@ def get_corrected_free_energy(free_energy, frequencies, frequency_cutoff=100.0,
     entropy_correction = (entropy_uncorrected - entropy_corrected)*temperature/1000.0
     corrected_free_energy = free_energy + entropy_correction
     return corrected_free_energy
+
+def numpy_to_bytes(arr):
+    """ Utility function for pickling numpy arrays """
+    arr_bytes = BytesIO()
+    np.save(arr_bytes, arr, allow_pickle=True)
+    arr_bytes = arr_bytes.getvalue()
+    return arr_bytes
+
+def bytes_to_numpy(arr_bytes):
+    """ Utility function for unpickling numpy arrays """
+    load_bytes = BytesIO(arr_bytes)
+    loaded_np = np.load(load_bytes, allow_pickle=True)
+    return loaded_np
+
+def compute_mass_spectrum(formula_dict, **kwargs):
+    """
+    Computes the expected low-res mass spec ions for a given formula.
+
+    Args:
+        formula dict (dict): e.g. {"C": 6, "H": 6}
+
+    Returns:
+        list of m/z ions
+        list of relative weights (out of 1 total)
+    """
+    form_vec = np.zeros(shape=92, dtype=np.int8)
+    for z, n in formula_dict.items():
+        if isinstance(z, str):
+            z = get_number(z)
+        assert isinstance(z, int), "atomic number must be integer"
+        form_vec[z] += n
+
+    masses, weights = _recurse_through_formula(form_vec, [0], [1], **kwargs)
+
+    new_masses, indices = np.unique(np.round(masses, decimals=1), return_inverse=True)
+    new_weights = np.zeros_like(new_masses)
+    for k in range(len(new_weights)):
+        new_weights[k] = np.sum(weights[np.nonzero(indices == k)])
+    new_weights = new_weights / np.max(new_weights)
+
+    return new_masses, new_weights
+
+def _recurse_through_formula(formula, masses, weights, cutoff=0.0000001, mass_precision=4, weight_precision=8):
+    """
+    Recurses through a formula and generates m/z isotopic pattern using tail recursion.
+
+    To prevent blowup of memory, fragments with very low abundance are ignored. Masses and weights are also rounded after every step.
+    To prevent error accumulation, internal precisions several orders of magnitude lower than the precision of interest should be employed.
+    The default values should work nicely for low-res MS applications.
+
+    Args:
+        formula (np.ndarray, dtype=np.int8): vector containing atoms left to incorporate. first element should always be 0 as there is no element 0.
+        masses (np.ndarray): list of mass fragments at current iteration
+        weights (np.ndarray): relative weights at current iteration
+        cutoff (float): cutoff for similarity (masses within ``cutoff`` will be combined)
+        mass_precision (int): number of decimal places to store for mass
+        weight_precision (int): number of decimal places to store for weight
+
+    Returns:
+        masses
+        weights
+    """
+    # check how many elements we haven't recursed thru yet
+    if np.array_equal(formula, np.zeros(shape=92, dtype=np.int8)):
+        return masses[np.argsort(masses)], weights[np.argsort(masses)]
+
+    # get masses/weights for current element
+    current_e = np.nonzero(formula)[0][0]
+    e_masses, e_weights = get_isotopic_distribution(current_e)
+
+    # combinatorially add the new masses and weights to our current lists
+    new_masses = np.zeros(shape=(len(masses)*len(e_masses)))
+    new_weights = np.zeros(shape=(len(masses)*len(e_masses)))
+    for i in range(len(masses)):
+        for j in range(len(e_masses)):
+            new_masses[i*len(e_masses)+j] = masses[i] + e_masses[j]
+            new_weights[i*len(e_masses)+j] = weights[i] * e_weights[j]
+
+    # delete duplicates and adjust weights (complicated)
+    newer_masses, indices = np.unique(np.round(new_masses, decimals=mass_precision), return_inverse=True)
+    newer_weights = np.zeros_like(newer_masses)
+    for k in range(len(newer_weights)):
+        newer_weights[k] = np.sum(new_weights[np.nonzero(indices == k)])
+    newer_weights = np.round(newer_weights, decimals=weight_precision)
+
+    # prune the low-abundance masses/weights and move on to the next element
+    formula[current_e] += -1
+    above_cutoff = np.nonzero(newer_weights > cutoff)
+    return _recurse_through_formula(formula, newer_masses[above_cutoff], newer_weights[above_cutoff], cutoff, mass_precision, weight_precision)
+
+def formula_dict_from_string(formula_string):
+    """
+    Eugene challenged me to code golf, this isn't my fault.
+
+    Args:
+        formula_string (str): the formula as a string, e.g. C10H12N2O1. you need the "1" explicitly
+
+    Returns:
+        formula_dict (dict): e.g. {'C': 10, 'H': 12, 'N': 2, 'O': 1}
+    """
+    return {t[0]: int(t[1]) for t in re.findall(r"([a-z]+)([0-9]+)", formula_string, re.I)}
