@@ -2,6 +2,8 @@ import math, copy, re
 import numpy as np
 import networkx as nx
 from scipy.spatial.distance import cdist
+import pkg_resources
+import yaml
 
 import cctk
 from cctk.helper_functions import (
@@ -14,8 +16,9 @@ from cctk.helper_functions import (
     compute_unit_vector,
     get_covalent_radius,
     get_vdw_radius,
-    get_isotopic_distribution,
-    compute_chirality,
+    numpy_to_bytes,
+    bytes_to_numpy,
+    _recurse_through_formula,
 )
 import cctk.topology as top
 
@@ -115,6 +118,30 @@ class Molecule:
         else:
             return f"Molecule ({len(self.atomic_numbers)} atoms)"
 
+    def __repr__(self):
+        return str(self) # placeholder
+
+#    def __eq__(self, other):
+    @classmethod
+    def equal(cls, mol1, mol2):
+        """
+        Atomic numbers, geometry, charge, and multiplicity all must match. Name is irrelevant.
+        """
+        if not isinstance(mol1, cctk.Molecule):
+            return False
+
+        if not isinstance(mol2, cctk.Molecule):
+            return False
+
+        comparisons = [
+            np.array_equal(mol1.atomic_numbers, mol2.atomic_numbers),
+            np.array_equal(mol1.geometry, mol2.geometry),
+            mol1.charge == mol2.charge,
+            mol1.multiplicity == mol2.multiplicity
+        ]
+
+        return all(comparisons)
+
     def assign_connectivity(self, cutoff=0.2, periodic_boundary_conditions=None):
         """
         Automatically recalculates bonds based on covalent radii. If two atoms are closer than the sum of their covalent radii + ``cutoff`` Angstroms,
@@ -283,14 +310,14 @@ class Molecule:
             elements = list(formula_dict.keys())
 
             #### H and C always come first
-            if "H" in elements:
-                elements.remove("H")
-                formula += f"H{formula_dict['H']}"
-
             if "C" in elements:
                 elements.remove("C")
                 formula += f"C{formula_dict['C']}"
 
+            if "H" in elements:
+                elements.remove("H")
+                formula += f"H{formula_dict['H']}"
+
             for element in sorted(elements):
                 formula += f"{element}{formula_dict[element]}"
 
@@ -705,60 +732,17 @@ class Molecule:
 
         return self
 
-    def _recurse_through_formula(self, formula, masses, weights, cutoff=0.0000001, mass_precision=4, weight_precision=8):
-        """
-        Recurses through a formula and generates m/z isotopic pattern using tail recursion.
-
-        To prevent blowup of memory, fragments with very low abundance are ignored. Masses and weights are also rounded after every step.
-        To prevent error accumulation, internal precisions several orders of magnitude lower than the precision of interest should be employed.
-        The default values should work nicely for low-res MS applications.
-
-        Args:
-            formula (np.ndarray, dtype=np.int8): vector containing atoms left to incorporate
-            masses (np.ndarray): list of mass fragments at current iteration
-            weights (np.ndarray): relative weights at current iteration
-            cutoff (float): cutoff for similarity (masses within ``cutoff`` will be combined)
-            mass_precision (int): number of decimal places to store for mass
-            weight_precision (int): number of decimal places to store for weight
-
-        Returns:
-            masses
-            weights
-        """
-        if np.array_equal(formula, np.zeros(shape=92, dtype=np.int8)):
-            return masses[np.argsort(masses)], weights[np.argsort(masses)]
-
-        current_e = np.nonzero(formula)[0][0]
-        e_masses, e_weights = get_isotopic_distribution(current_e)
-
-        new_masses = np.zeros(shape=(len(masses)*len(e_masses)))
-        new_weights = np.zeros(shape=(len(masses)*len(e_masses)))
-        for i in range(len(masses)):
-            for j in range(len(e_masses)):
-                new_masses[i*len(e_masses)+j] = masses[i] + e_masses[j]
-                new_weights[i*len(e_masses)+j] = weights[i] * e_weights[j]
-
-        newer_masses, indices = np.unique(np.round(new_masses, decimals=mass_precision), return_inverse=True)
-        newer_weights = np.zeros_like(newer_masses)
-        for k in range(len(newer_weights)):
-            newer_weights[k] = np.sum(new_weights[np.nonzero(indices == k)])
-        newer_weights = np.round(newer_weights, decimals=weight_precision)
-
-        formula[current_e] += -1
-        above_cutoff = np.nonzero(newer_weights > cutoff)
-        return self._recurse_through_formula(formula, newer_masses[above_cutoff], newer_weights[above_cutoff], cutoff, mass_precision, weight_precision)
-
     def calculate_mass_spectrum(self, **kwargs):
         """
-        Generates list of m/z values based on formula string (e.g. "C10H12")
+        Generates list of m/z values.
 
         Final weights rounded to one decimal point (because of low-res MS).
         """
         form_vec = np.zeros(shape=92, dtype=np.int8)
         for z in self.atomic_numbers:
-            form_vec[z - 1] += 1
+            form_vec[z] += 1
 
-        masses, weights = self._recurse_through_formula(form_vec, [0], [1], **kwargs)
+        masses, weights = _recurse_through_formula(form_vec, [0], [1], **kwargs)
 
         new_masses, indices = np.unique(np.round(masses, decimals=1), return_inverse=True)
         new_weights = np.zeros_like(new_masses)
@@ -1563,7 +1547,7 @@ class Molecule:
         try:
             url_name = re.sub(" ", "%20", name)
             url = 'http://cactus.nci.nih.gov/chemical/structure/' + url_name + '/smiles'
-            smiles = urlopen(url).read().decode('utf8')
+            smiles = urlopen(url, timeout=5).read().decode('utf8')
             return cls.new_from_smiles(smiles)
         except Exception as e:
             raise ValueError(f"something went wrong auto-generating molecule {name}:\nurl: {url}\n{e}")
@@ -1748,3 +1732,99 @@ class Molecule:
                 else:
                     return return_list
 
+
+    def to_string(self):
+        """
+        Save the current molecule as a string, for subsequent loading. Not human-readable.
+
+        Vibrational modes are currently not saved.
+        """
+        # name, charge, multiplicity need no encoding
+        atomic_number_encoding = numpy_to_bytes(self.atomic_numbers.view(np.ndarray))
+        geometry_encoding = numpy_to_bytes(self.geometry.view(np.ndarray))
+        bonds_encoding = numpy_to_bytes(nx.convert_matrix.to_numpy_array(self.bonds))
+
+        if self.name is None:
+            self.name = "name"
+
+        cctk_version = pkg_resources.get_distribution("cctk").version
+
+        store_dict = {
+            "name": self.name,
+            "charge": self.charge,
+            "multiplicity": self.multiplicity,
+            "atomic_numbers": atomic_number_encoding,
+            "geometry": geometry_encoding,
+            "bonds": bonds_encoding,
+            "cctk_version": cctk_version,
+        }
+
+        return yaml.dump(store_dict)
+
+    @classmethod
+    def from_string(cls, string, check_version=True):
+        """
+        Loads a ``cctk.Molecule`` object from a string.
+
+        Arguments:
+            string (str): stringified version of the molecule
+            check_version (bool): whether version consistency should be enforced
+        """
+
+        try:
+            store_dict = yaml.safe_load(string)
+
+            if check_version:
+                cctk_version = pkg_resources.get_distribution("cctk").version
+                assert cctk_version == store_dict["cctk_version"], f"Warning: the data was saved in cctk {store_dict['cctk_version']} but is being loaded in cctk {cctk_version}!"
+
+            atomic_numbers = bytes_to_numpy(store_dict["atomic_numbers"]).astype(np.int8)
+            geometry = bytes_to_numpy(store_dict["geometry"]).astype(np.float32)
+            bonds = nx.convert_matrix.from_numpy_array(bytes_to_numpy(store_dict["bonds"]))
+
+            mol = cls(
+                atomic_numbers,
+                geometry,
+                bonds=bonds,
+                charge=store_dict["charge"],
+                multiplicity=store_dict["multiplicity"],
+                name=store_dict["name"],
+                checks=False, # trust nx data implicitly
+            )
+
+            return mol
+
+        except Exception as e:
+            raise ValueError(f"this stringified Molecule fails import: {e}")
+
+    def coulomb_analysis(self, atoms1, atoms2, charges):
+        """
+        Computes the net Coulomb forces between atoms ``atoms1`` and atoms ``atoms2``.
+        """
+        if isinstance(charges, np.ndarray):
+            charges = charges.view(cctk.OneIndexedArray)
+        elif isinstance(charges, list):
+            charges = cctk.OneIndexedArray(charges)
+
+        assert isinstance(charges, cctk.OneIndexedArray), "charges must be cctk.OneIndexedArray"
+        assert len(charges) == self.num_atoms(), "need a charge for every atom"
+        assert isinstance(atoms1, list)
+        assert isinstance(atoms2, list)
+
+        q1 = charges[atoms1]
+        q2 = charges[atoms2]
+
+        # need to convert to Bohr
+        r1 = self.geometry[atoms1] / 0.529
+        r2 = self.geometry[atoms2] / 0.529
+
+        R = cdist(r1, r2)**2
+        Q = np.outer(q1, q2)
+
+        energy = 0
+        for i in range(len(atoms1)):
+            assert atoms1[i] not in atoms2, "lists must be non-overlapping"
+            for j in range(len(atoms2)):
+                energy += Q[i][j] / R[i][j]
+
+        return energy * 627.509 # convert to kcal/mol
