@@ -127,7 +127,9 @@ def read_file_fast(file_text, filename, link1idx, max_len=20000, extended_opt_in
 
             # special geometry handling :/
             if idx == 3:
-                if len(block_matches[3]) == len(word_matches[0]):
+                # ccw 10.8.2021 - changed "==" to "<=" to prevent issues where # geoms would get stuck.
+                # can't remember quite why this was needed. hopefully it is ok this way. tests pass.
+                if len(block_matches[3]) <= len(word_matches[0]):
                     block_matches[3].append(match)
                 else:
                     block_matches[3][-1] = match
@@ -187,8 +189,11 @@ def read_file_fast(file_text, filename, link1idx, max_len=20000, extended_opt_in
             max_int = extract_parameter(word_matches[10], 3)
             delta_e = extract_parameter(word_matches[11], 3, cast_to_float=False)
 
-        for idx, force in enumerate(rms_forces):
-            properties[idx]["rms_force"] = force
+        # ccw 10.8.2021 - ad hoc correction to Gaussian. unsure what's going on here. sometimes len(rms_forces) > len(g)
+        force_property_index = min(len(g), len(rms_forces))
+
+        for idx in range(force_property_index):
+            properties[idx]["rms_force"] = rms_forces[idx]
             properties[idx]["rms_displacement"] = rms_disp[idx]
 
             if extended_opt_info:
@@ -214,7 +219,7 @@ def read_file_fast(file_text, filename, link1idx, max_len=20000, extended_opt_in
                     change_in_energy = re.sub(r"Energy=", "", delta_e[idx])
                     properties[idx]["predicted_change_in_energy"] = float(change_in_energy.replace('D', 'E'))
 
-    if cctk.GaussianJobType.FREQ in job_types:
+    if cctk.GaussianJobType.FREQ in job_types and len(molecules):
         enthalpies = extract_parameter(word_matches[12], 6)
         if len(enthalpies) == 1:
             properties[-1]["enthalpy"] = enthalpies[0]
@@ -265,24 +270,28 @@ def read_file_fast(file_text, filename, link1idx, max_len=20000, extended_opt_in
             if couplings is not None:
                 properties[-1]["j_couplings"] = couplings
 
-    if cctk.GaussianJobType.FORCE in job_types:
+    if cctk.GaussianJobType.FORCE in job_types and len(molecules):
         assert len(molecules) == 1, "force jobs should not be combined with optimizations!"
-        forces = parse_forces(block_matches[7])
+        force_block = block_matches[7]
+        if len(force_block) == 0:
+            raise ValueError("no forces to parse!")
+        forces = parse_forces(force_block)
         properties[0]["forces"] = forces
 
-    if cctk.GaussianJobType.POP in job_types:
+    if cctk.GaussianJobType.POP in job_types and len(molecules):
         if re.search("hirshfeld", f.route_card) or re.search("cm5", f.route_card) and len(block_matches[8]) > 0:
             charges, spins = parse_hirshfeld(block_matches[8])
             properties[-1]["hirshfeld_charges"] = charges
             properties[-1]["hirshfeld_spins"] = spins
 
-    try:
-        charges, dipole, dipole_v = parse_charges_dipole(block_matches[9], block_matches[10])
-        properties[-1]["mulliken_charges"] = charges
-        properties[-1]["dipole_moment"] = dipole
-        properties[-1]["dipole_vector"] = dipole_v
-    except Exception as e:
-        pass
+    if len(molecules):
+        try:
+            charges, dipole, dipole_v = parse_charges_dipole(block_matches[9], block_matches[10])
+            properties[-1]["mulliken_charges"] = charges
+            properties[-1]["dipole_moment"] = dipole
+            properties[-1]["dipole_vector"] = dipole_v
+        except Exception as e:
+            pass
 
     for mol, prop in zip(molecules, properties):
         f.ensemble.add_molecule(mol, properties=prop)
@@ -404,7 +413,14 @@ def extract_parameter(lines, position, cast_to_float=True):
 
 def parse_forces(force_block):
     forces = []
-    for line in force_block[0].split("\n")[2:]:
+    try:
+        split_block = force_block[0].split("\n")[2:]
+    except Exception as e:
+        print(e)
+        print("------force block-------")
+        print(force_block)
+        raise e
+    for line in split_block:
         fields = re.split(" +", line)
         fields = list(filter(None, fields))
 
@@ -700,7 +716,7 @@ def read_nmr_shifts(blocks, num_atoms):
         tensor[2][2] = float(re.search("ZZ=\s+(?P<val>-?\d+\.\d+)", block).group("val"))
         tensors.append(tensor)
 
-    if len(shieldings) is not 0:
+    if len(shieldings) != 0:
         assert len(shieldings) == num_atoms, f"Expected {num_atoms} shieldings but found {len(shieldings)}!"
         for shielding, tensor in zip(shieldings, tensors):
             assert 0.01 > abs(np.trace(tensor)/3 - shielding)
