@@ -176,10 +176,17 @@ class TestGaussian(unittest.TestCase):
         self.assertTrue(-76.20098 - emp4 < 0.0001)
 
     def test_pathological(self):
-#        path = "test/static/sugar_failed.out"
-#        file = cctk.GaussianFile.read_file(path)
-#        self.assertTrue(isinstance(file, cctk.GaussianFile))
-        pass
+        path = "test/static/cation_cl.out"
+        file = cctk.GaussianFile.read_file(path)
+        self.assertTrue(isinstance(file, cctk.GaussianFile))
+
+        path = "test/static/cation_cl2.out"
+        file = cctk.GaussianFile.read_file(path)
+        self.assertTrue(isinstance(file, cctk.GaussianFile))
+
+        path = "test/static/cation_cl3.out"
+        file = cctk.GaussianFile.read_file(path)
+        self.assertTrue(isinstance(file, cctk.GaussianFile))
 
 if __name__ == '__main__':
     unittest.main()
