@@ -24,6 +24,9 @@ class TestNMR(unittest.TestCase):
         tensors = ensemble[:, "shielding_tensors"]
         self.assertEqual(tensors[0][0][0], 198.2259)
         self.assertEqual(tensors[1][0][0], 32.6869)
+        for shielding, tensor in zip(shieldings, tensors):
+            pred_shielding = np.trace(tensor)/3
+            self.assertTrue((shielding-pred_shielding < 0.001))
 
     def test_nmr2(self):
         # this file contains opt freq followed by Link1 NMR on methane
@@ -41,6 +44,11 @@ class TestNMR(unittest.TestCase):
         shieldings = list(ensemble[-1,"isotropic_shielding"])
         self.assertListEqual(list(shieldings), [192.9242, 31.8851, 31.8851, 31.8851, 31.8851])
 
+        tensors = ensemble[:, "shielding_tensors"]
+        for shielding, tensor in zip(shieldings, tensors):
+            pred_shielding = np.trace(tensor)/3
+            self.assertTrue((shielding-pred_shielding < 0.001))
+
     def test_nmr3(self):
         # this file contains opt freq / Link1 NMR on ethane then Link1 single point NMR on methane
         gaussian_file = cctk.GaussianFile.read_file("test/static/ethane.out")
@@ -90,6 +98,11 @@ class TestNMR(unittest.TestCase):
         self.assertTrue((np.abs(scaled_shifts[0] - expected_shifts) <= 0.001).all())
         #print(shift_labels)
 
+        tensors = ensemble[:, "shielding_tensors"]
+        for shielding, tensor in zip(shieldings, tensors):
+            pred_shielding = np.trace(tensor)/3
+            self.assertTrue((shielding-pred_shielding < 0.001))
+
     def test_nmr5(self):
         # ensure all acetone files can be read
         for idx in range(1,7):
